// brain.def -- the definitions for the static (class) variables for the
// brain classes -- include this in your main .cc file.


//  brain-objs.hh -------------

// const Braintype Brain::bt = BRAIN; 

// const Braintype Sense::bt = SENSE;   
braintrie Sense::senses;
bool add_sense(char * name, bool (*value)());

// const Braintype Act::bt = ACT; 
braintrie Act::actions;
// bool Act::add_act(char * name, lapres (*value)());

// const Braintype Trig::bt = TRIG; 

// const Braintype Lap::bt = LAP;    
braintrie Lap::Laps;
// void Lap::add_lap(char *myname, Brain ** mylap, Trig *mytrig);
// Lap * Lap::get_lap (char *key);
// Lap * Lap::find_lap (char *key);

// const Braintype Comp::bt = COMP;    
braintrie Comp::Comps;
// void Comp::add_comp(char *myname, char *lapstring, Trig *mytrig);
// Comp * Comp::get_comp (char *key);
// Comp * Comp::find_comp (char *key);

// const Braintype Drive::bt = DRIVE;    
// void Drive::add_drive(char *myname, char *lapstring, Trig *mytrig);
// Drive * Drive::get_drive (char *key);
// Drive * Drive::find_drive (char *key);

