/* monx.hh - declarations for functions that aren't in classes
   (general utilities, I hope!)

   $Log: monx.hh,v $
   Revision 1.1  1999/12/27 23:48:46  joanna
   Initial revision

   Revision 1.3  1997/09/10 01:36:43  joanna
   works with robs stuff -- fixed loads more chaining problems,
   new rob libraries.  Interactive rob ringrose, not piped yet.

   Revision 1.1  1997/08/10 18:34:11  joanna
   Initial revision

*/

#include <math.h>

// #define SPEW    1            // save stuff to log files

void Usage(const char*);
// void initclasstypes();
void initprims();
void read_brains(char * filename, char * pathname);


#define NAMESIZE 48

// I think c++ is supposed to define these?

// enum bool {false, true, error};  // :-) :-) :-)

// enum lapres {CONTINUE, HABCONT, SUCCESS, FAILURE};  Grr...

#define FAILURE 0
#define SUCCESS 1
#define CONTINUE 2
// just for drives habituating competences --
// a competence chunk / lap is done but the competence itself isn't done...
#define HABCONT 3
#define lapres int

// where things go...

extern char *LOG_PATH;
extern char *IOF_PATH;	// lap and landmark files

// Ignore the bottom signal bit of system and just get the processes
// return value.


extern "C" {
  int system(const char *string);
}

inline int sysret(char *com) { return (system(com) >> 8);}


