// monx.def -- the definitions for the static (class) variables in the monx
// library


//  monx-dirattn.hh

char * Dirattn::attn = 0;
Dirattn * Dirattn::directories = 0;

// lapres Dirattn::cd_attn();
// lapres Dirattn::attn_to_dir();
// bool Dirattn::attend_dir();
