// brain-read.hh -- just a few function decs

/*
  $Log: brain-read.hh,v $
  Revision 1.1  1999/12/27 23:48:46  joanna
  Initial revision

  Revision 1.3  1997/09/10 01:36:43  joanna
  works with robs stuff -- fixed loads more chaining problems,
  new rob libraries.  Interactive rob ringrose, not piped yet.

  Revision 1.1  1997/08/10 18:34:11  joanna
  Initial revision


*/

void read_brains(char * filename, char * pathname);
void read_lap(char * name, char * stuff);
void getlist (char *list, char *stuff, int &place);
Brain ** make_primarray(char * stuff);

// void INTerface(int);


